/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Certus SDK, Version 7.0                    */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Raw Satellite Ephemeris */
public class ANPacket61
{
	public static final int LENGTH_GPS		= 132;
	public static final int LENGTH_GLONASS	= 94;
	public static final int LENGTH_GPS_IONO	= 92;
	
	public static final int SATELLITE_SYSTEM_UNKNOWN 	= 0;
	public static final int SATELLITE_SYSTEM_GPS 		= 1;
	public static final int SATELLITE_SYSTEM_GLONASS 	= 2;
	public static final int SATELLITE_SYSTEM_BEIDOU 	= 3;
	public static final int SATELLITE_SYSTEM_GALILEO 	= 4;
	public static final int SATELLITE_SYSTEM_SBAS 		= 5;
	public static final int SATELLITE_SYSTEM_QZSS 		= 6;
	public static final int SATELLITE_SYSTEM_STARFIRE 	= 7;
	public static final int SATELLITE_SYSTEM_OMNISTAR 	= 8;

	public long unixTime;
	public int satelliteSystem;
	public int satelliteNumber;
	public GPSEphemeris gpsEphemeris = null;
	public GlonassEphemeris glonassEphemeris = null;
	public GPSIonosphericUTC gpsIonosphericUTC = null;

	public ANPacket61(ANPacket packet)
	{
		unixTime = TypeConversion.bytesToUInt32(packet.data, 0);
		satelliteSystem = packet.data[4] & 0xFF;
		satelliteNumber = packet.data[5] & 0xFF;
		if (satelliteSystem == SATELLITE_SYSTEM_GPS)
		{
			if(packet.length == LENGTH_GPS)
			{
				gpsEphemeris = new GPSEphemeris(packet);
			}
			else if(packet.length == LENGTH_GPS_IONO)
			{
				gpsIonosphericUTC = new GPSIonosphericUTC(packet);
			}
		}
		else if (satelliteSystem == SATELLITE_SYSTEM_GLONASS && packet.length == LENGTH_GLONASS)
		{
			glonassEphemeris = new GlonassEphemeris(packet);
		}
	}

	public class GPSEphemeris
	{
		public long toe;
		public int iodc;
		public int iode;
		public double af0;
		public double af1;
		public double af2;
		public double crs;
		public double deltaN;
		public double m0;
		public double cuc;
		public double e;
		public double cus;
		public double sqrtA;
		public double cic;
		public double omega0;
		public double cis;
		public double i0;
		public double crc;
		public double omega;
		public double omegaDot;
		public double idot;
		public double tgd;
		public int gpsWeek;
		public long tow;
		public int ura;
		public Boolean l2PData;
		public Boolean l2Codes;
		public Boolean antiSpoofing;
		public int satelliteHealth;
		public Boolean fitInterval;
		public Boolean uraBad;
		public int satelliteType;
		
		public GPSEphemeris()
		{
		}

		public GPSEphemeris(ANPacket packet)
		{
			toe = TypeConversion.bytesToUInt32(packet.data, 6);
			iodc = TypeConversion.bytesToUInt16(packet.data, 10);
			iode = TypeConversion.bytesToUInt16(packet.data, 12);
			af0 = TypeConversion.bytesToFloat(packet.data, 14);
			af1 = TypeConversion.bytesToFloat(packet.data, 18);
			af2 = TypeConversion.bytesToFloat(packet.data, 22);
			crs = TypeConversion.bytesToFloat(packet.data, 26);
			deltaN = TypeConversion.bytesToFloat(packet.data, 30);
			m0 = TypeConversion.bytesToDouble(packet.data, 34);
			cuc = TypeConversion.bytesToFloat(packet.data, 42);
			e = TypeConversion.bytesToDouble(packet.data, 46);
			cus = TypeConversion.bytesToFloat(packet.data, 54);
			sqrtA = TypeConversion.bytesToDouble(packet.data, 58);
			cic = TypeConversion.bytesToFloat(packet.data, 66);
			omega0 = TypeConversion.bytesToDouble(packet.data, 70);
			cis = TypeConversion.bytesToFloat(packet.data, 78);
			i0 = TypeConversion.bytesToDouble(packet.data, 82);
			crc = TypeConversion.bytesToFloat(packet.data, 90);
			omega = TypeConversion.bytesToDouble(packet.data, 94);
			omegaDot = TypeConversion.bytesToDouble(packet.data, 102);
			idot = TypeConversion.bytesToDouble(packet.data, 110);
			tgd = TypeConversion.bytesToFloat(packet.data, 118);
			gpsWeek = TypeConversion.bytesToUInt16(packet.data, 122);
			tow = TypeConversion.bytesToUInt32(packet.data, 124);
			ura = TypeConversion.bytesToUInt16(packet.data, 128);
			l2PData = (packet.data[130] & 0x01) > 0;
			l2Codes = (packet.data[130] & 0x02) > 0;
			antiSpoofing = (packet.data[130] & 0x04) > 0;
			satelliteHealth = (packet.data[131] & 0x03) << 4 | (packet.data[130] >>> 4) & 0x0F;
			fitInterval = (packet.data[131] & 0x04) > 0;
			uraBad = (packet.data[131] & 0x08) > 0;
			satelliteType = (packet.data[131] >> 4) & 0x07;
		}
	}

	public class GlonassEphemeris
	{
		public double clockBias;
		public double frequencyBias;
		public double position[];
		public double velocity[];
		public double acceleration[];
		public long frameStartTime;
		public int age;
		public int frequencySlot;
		public int satelliteHealth;
		
		public GlonassEphemeris()
		{
			position = new double[3];
			velocity = new double[3];
			acceleration = new double[3];
		}

		public GlonassEphemeris(ANPacket packet)
		{
			clockBias = TypeConversion.bytesToFloat(packet.data, 6);
			frequencyBias = TypeConversion.bytesToFloat(packet.data, 10);
			position = new double[3];
			position[0] = TypeConversion.bytesToDouble(packet.data, 14);
			position[1] = TypeConversion.bytesToDouble(packet.data, 22);
			position[2] = TypeConversion.bytesToDouble(packet.data, 30);
			velocity = new double[3];
			velocity[0] = TypeConversion.bytesToDouble(packet.data, 38);
			velocity[1] = TypeConversion.bytesToDouble(packet.data, 46);
			velocity[2] = TypeConversion.bytesToDouble(packet.data, 54);
			acceleration = new double[3];
			acceleration[0] = TypeConversion.bytesToDouble(packet.data, 62);
			acceleration[1] = TypeConversion.bytesToDouble(packet.data, 70);
			acceleration[2] = TypeConversion.bytesToDouble(packet.data, 78);
			frameStartTime = TypeConversion.bytesToUInt32(packet.data, 86);
			age = packet.data[90] & 0xFF;
			frequencySlot = packet.data[91] & 0xFF;
			satelliteHealth = packet.data[92] & 0xFF;
		}
	}

	public class GPSIonosphericUTC
	{
		public double[] alpha;
		public double[] beta;
		public double[] asub;
		public int deltatls; // current leap seconds
		public int deltatlsf; // next leap seconds
		public int wnsublsf; // next leap seconds week
		public int dn; // next leap seconds day of week (1..7 where 1=Sunday and 7=Saturday)
		
		public GPSIonosphericUTC()
		{
			alpha = new double[4];
			beta = new double[4];
			asub = new double[2];
		}
		
		public GPSIonosphericUTC(ANPacket packet)
		{
			alpha = new double[4];
			alpha[0] = TypeConversion.bytesToDouble(packet.data, 6);
			alpha[1] = TypeConversion.bytesToDouble(packet.data, 14);
			alpha[2] = TypeConversion.bytesToDouble(packet.data, 22);
			alpha[3] = TypeConversion.bytesToDouble(packet.data, 30);
			beta = new double[4];
			beta[0] = TypeConversion.bytesToDouble(packet.data, 38);
			beta[1] = TypeConversion.bytesToDouble(packet.data, 46);
			beta[2] = TypeConversion.bytesToDouble(packet.data, 54);
			beta[3] = TypeConversion.bytesToDouble(packet.data, 62);
			asub = new double[2];
			asub[0] = TypeConversion.bytesToDouble(packet.data, 70);
			asub[1] = TypeConversion.bytesToDouble(packet.data, 78);
			deltatls = TypeConversion.bytesToInt16(packet.data, 86);
			deltatlsf = TypeConversion.bytesToInt16(packet.data, 88);
			wnsublsf = TypeConversion.bytesToUInt16(packet.data, 90);
			dn = packet.data[91] & 0xFF;
		}
	}
}
