/******************************************************************************/
/*                                                                            */
/*                Advanced Navigation Packet Protocol Library                 */
/*                   Java Language Certus SDK, Version 7.0                    */
/*                    Copyright 2023, Advanced Navigation                     */
/*                                                                            */
/******************************************************************************/
/*
 * Copyright (C) 2023 Advanced Navigation
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package com.advancednavigation.anPackets;

/* Extended Satellites */
public class ANPacket84
{
	public static final int SATELLITE_SYSTEM_UNKNOWN 	= 0;
	public static final int SATELLITE_SYSTEM_GPS 		= 1;
	public static final int SATELLITE_SYSTEM_GLONASS 	= 2;
	public static final int SATELLITE_SYSTEM_BEIDOU 	= 3;
	public static final int SATELLITE_SYSTEM_GALILEO 	= 4;
	public static final int SATELLITE_SYSTEM_SBAS 		= 5;
	public static final int SATELLITE_SYSTEM_QZSS 		= 6;
	public static final int SATELLITE_SYSTEM_STARFIRE 	= 7;
	public static final int SATELLITE_SYSTEM_OMNISTAR 	= 8;

	public static final int MAX_SATS = (255 - 2) / 9;
	public static final int MAX_PACKETS = 3;

	public int gpsSatellites;
	public int glonassSatellites;
	public int beidouSatellites;
	public int galileoSatellites;
	public int sbasSatellites;
	public int totalPackets;
	public int packetIndex; 	// 1 .. total_packets

	public SatelliteExt[] satellites;

	public ANPacket84(ANPacket packet)
	{
		gpsSatellites = 0;
		glonassSatellites = 0;
		sbasSatellites = 0;
		beidouSatellites = 0;
		galileoSatellites = 0;

		totalPackets = packet.data[0] & 0x0FF;
		packetIndex = packet.data[1] & 0x0FF;

		int satellitesCount = (packet.length - 2) / 9;

		if ((packet.length > 255) || (packet.length % 9 != 2))
		{
			System.out.printf("Error: ANPacket84 invalid length %d.\n", packet.length);
			totalPackets = 0;
			packetIndex = 0;
			return;
		}
		if ((totalPackets > MAX_PACKETS) || (packetIndex > totalPackets) || (packetIndex < 1) || (totalPackets < 1))
		{
			System.out.printf("Error: ANPacket84 bad packet num %d/%d\n", packetIndex, totalPackets);
			totalPackets = 0;
			packetIndex = 0;
			return;
		}

		satellites = new SatelliteExt[satellitesCount];

		for (int i = 0; i < satellitesCount; i++)
		{
			satellites[i] = new SatelliteExt();
			satellites[i].navigationSystem = packet.data[2 + 9 * i] & 0x0FF;
			satellites[i].prn = packet.data[2 + 9 * i + 1] & 0x0FF;
			byte freqs = (byte) (packet.data[2 + 9 * i + 2] & 0xFF);
			if ((freqs & 0x01) != 0) satellites[i].l1_ca = true;
			if ((freqs & 0x02) != 0) satellites[i].l1_c  = true;
			if ((freqs & 0x04) != 0) satellites[i].l1_p  = true;
			if ((freqs & 0x08) != 0) satellites[i].l1_m  = true;
			if ((freqs & 0x10) != 0) satellites[i].l2_c  = true;
			if ((freqs & 0x20) != 0) satellites[i].l2_p  = true;
			if ((freqs & 0x40) != 0) satellites[i].l2_m  = true;
			if ((freqs & 0x80) != 0) satellites[i].l5    = true;
			satellites[i].elevation = packet.data[2 + 9 * i + 3] & 0x0FF;
			satellites[i].azimuth = TypeConversion.bytesToUInt16(packet.data, 2 + 9 * i + 4);
			satellites[i].primary_snr   = packet.data[2 + 9 * i + 6] & 0xFF;
			satellites[i].secondary_snr = packet.data[2 + 9 * i + 7] & 0xFF;
			byte flags = (byte) (packet.data[2 + 9 * i + 8] & 0xFF);
			if ((flags & 0x01) != 0) satellites[i].visible_receiver_1 = true;
			if ((flags & 0x02) != 0) satellites[i].visible_receiver_2 = true;
			if ((flags & 0x04) != 0) satellites[i].used_in_primary_position = true;
			if ((flags & 0x08) != 0) satellites[i].used_in_moving_baseline = true;

			if (satellites[i].navigationSystem == 1) gpsSatellites++;
			else if (satellites[i].navigationSystem == 2) glonassSatellites++;
			else if (satellites[i].navigationSystem == 3) beidouSatellites++;
			else if (satellites[i].navigationSystem == 4) galileoSatellites++;
			else if (satellites[i].navigationSystem == 5) sbasSatellites++;
		}
	}

	public static class SatelliteExt
	{
		public int navigationSystem;
		public int prn;
		public boolean l1_ca = false;
		public boolean l1_c = false;
		public boolean l1_p = false;
		public boolean l1_m = false;
		public boolean l2_c = false;
		public boolean l2_p = false;
		public boolean l2_m = false;
		public boolean l5 = false;
		public int elevation;
		public int azimuth;
		public int primary_snr;
		public int secondary_snr;
		public boolean visible_receiver_1;
		public boolean visible_receiver_2;
		public boolean used_in_primary_position;
		public boolean used_in_moving_baseline;

		public String navigationSystemString()
		{
			switch (navigationSystem)
			{
				case SATELLITE_SYSTEM_GPS:
					return "GPS";
				case SATELLITE_SYSTEM_GLONASS:
					return "GLONASS";
				case SATELLITE_SYSTEM_BEIDOU:
					return "BeiDou";
				case SATELLITE_SYSTEM_GALILEO:
					return "Galileo";
				case SATELLITE_SYSTEM_SBAS:
					return "SBAS";
				case SATELLITE_SYSTEM_QZSS:
					return "QZSS";
				case SATELLITE_SYSTEM_STARFIRE:
					return "StarFire";
				case SATELLITE_SYSTEM_OMNISTAR:
					return "Omnistar";
				default:
					return "";
			}
		}

		public String frequenciesString()
		{
			String frequencies = "";
			if (l1_ca)
			{
				if (navigationSystem == ANPacket60.SATELLITE_SYSTEM_BEIDOU)
				{
					frequencies += "B1 ";
				}
				else if (navigationSystem == ANPacket60.SATELLITE_SYSTEM_GALILEO)
				{
					frequencies += "E1OS ";
				}
				else frequencies += "L1C/A ";
			}
			if (l1_c) frequencies += "L1C ";
			if (l1_p) frequencies += "L1P ";
			if (l1_m) frequencies += "L1M ";
			if (l2_c)
			{
				if (navigationSystem == ANPacket60.SATELLITE_SYSTEM_BEIDOU)
				{
					frequencies += "B2 ";
				}
				else frequencies += "L2C ";
			}
			if (l2_p) frequencies += "L2P ";
			if (l2_m) frequencies += "L2M ";
			if (l5)
			{
				if (navigationSystem == ANPacket60.SATELLITE_SYSTEM_BEIDOU)
				{
					frequencies += "B3 ";
				}
				else if (navigationSystem == ANPacket60.SATELLITE_SYSTEM_GALILEO)
				{
					frequencies += "E5 ";
				}
				else frequencies += "L5 ";
			}
			return frequencies;
		}
	}
}
